@@ -2,15 +2,15 @@ import logging
 import os
 from contextlib import contextmanager
 
-import yaml
 from funcy import first
 from voluptuous import Invalid
 
 from dvc.cache.local import _log_exceptions
+from dvc.exceptions import DvcException
 from dvc.schema import COMPILED_LOCK_FILE_STAGE_SCHEMA
 from dvc.utils import dict_sha256, relpath
 from dvc.utils.fs import makedirs
-from dvc.utils.yaml import dump_yaml
+from dvc.utils.serialize import YAMLFileCorruptedError, dump_yaml, load_yaml
 
 from .loader import StageLoader
 from .serialize import to_single_stage_lockfile
@@ -18,6 +18,11 @@ from .serialize import to_single_stage_lockfile
 logger = logging.getLogger(__name__)
 
 
+class RunCacheNotFoundError(DvcException):
+    def __init__(self, stage):
+        super().__init__(f"No run-cache for {stage.addressing}")
+
+
 def _get_cache_hash(cache, key=False):
     if key:
         cache["outs"] = [out["path"] for out in cache.get("outs", [])]
@@ -29,7 +34,7 @@ def _get_stage_hash(stage):
         return None
 
     for dep in stage.deps:
-        if not (dep.scheme == "local" and dep.def_path and dep.get_checksum()):
+        if not (dep.scheme == "local" and dep.def_path and dep.get_hash()):
             return None
 
     for out in stage.outs:
@@ -54,11 +59,10 @@ class StageCache:
         path = self._get_cache_path(key, value)
 
         try:
-            with open(path) as fobj:
-                return COMPILED_LOCK_FILE_STAGE_SCHEMA(yaml.safe_load(fobj))
+            return COMPILED_LOCK_FILE_STAGE_SCHEMA(load_yaml(path))
         except FileNotFoundError:
             return None
-        except (yaml.error.YAMLError, Invalid):
+        except (YAMLFileCorruptedError, Invalid):
             logger.warning("corrupted cache file '%s'.", relpath(path))
             os.unlink(path)
             return None
@@ -80,7 +84,7 @@ class StageCache:
         return None
 
     def _create_stage(self, cache, wdir=None):
-        from . import create_stage, PipelineStage
+        from . import PipelineStage, create_stage
 
         stage = create_stage(
             PipelineStage,
@@ -144,18 +148,34 @@ class StageCache:
         makedirs(dpath, exist_ok=True)
         dump_yaml(path, cache)
 
-    def is_cached(self, stage):
-        return bool(self._load(stage))
-
-    def restore(self, stage):
+    def _restore(self, stage):
+        stage.save_deps()
         cache = self._load(stage)
         if not cache:
-            return
-        StageLoader.fill_from_lock(stage, cache)
+            raise RunCacheNotFoundError(stage)
 
+        StageLoader.fill_from_lock(stage, cache)
         for out in self._uncached_outs(stage, cache):
             out.checkout()
 
+        if not stage.outs_cached():
+            raise RunCacheNotFoundError(stage)
+
+    def restore(self, stage, run_cache=True):
+        if stage.is_callback or stage.always_changed:
+            raise RunCacheNotFoundError(stage)
+
+        if not stage.already_cached():
+            if not run_cache:  # backward compatibility
+                raise RunCacheNotFoundError(stage)
+            self._restore(stage)
+
+        logger.info(
+            "Stage '%s' is cached - skipping run, checking out outputs",
+            stage.addressing,
+        )
+        stage.checkout()
+
     @staticmethod
     def _transfer(func, from_remote, to_remote):
         ret = []
